<?php

namespace WPIS;

class EstatePostType extends EstateProperties {

    static public $postTypeName = 'wpis_estates';
    static public $postTypeMetaPrefix = 'wpis';
    static public $taxonomies;

    static public function init() {

        $class = new self();

        if( SettingsManager::get('enableImportCustomPostType') ){
            self::$postTypeName = SettingsManager::get('customPostType');
        }
        else {
            add_action( 'init', [$class, 'registerPostType']);
            add_filter( 'add_menu_classes', [$class, 'addMenuPostCount']);
            add_filter( 'manage_' . self::$postTypeName .  '_posts_columns', [$class, 'setCustomColumns'] );
            add_action( 'manage_' . self::$postTypeName .  '_posts_custom_column' , [$class, 'applyCustomColumns'], 10, 2 );
            add_action( 'add_meta_boxes', [$class, 'createMetabox']);
            add_action( 'save_post', [$class, 'saveMetaboxData'], 10, 2);
        }

        if( SettingsManager::get('enableTaxonomies') ){
            $class->initTaxonomies();
            add_action( 'init' , [$class, 'registerTaxonomies'] );
        }




        add_action( 'init', [$class, 'handleRedirection' ]);

    }


    public function registerPostType(){

        register_post_type(
            self::$postTypeName,
            [
                'labels'      => [
                    'name'          => __('Propriétés', 'wpis'),
                    'singular_name' => __('Propriété', 'wpis'),
                ],
                'public'       => true,
                'has_archive'  => true,
                'show_in_rest' => false,
                'rewrite'      => [
                    'slug' => SettingsManager::get('postTypeEstateSlug', self::$postTypeName)
                ],
                'menu_icon' => 'dashicons-admin-home',
                'supports'  => [
                    'title',
                    'editor',
                    'thumbnail'
                ]
            ]
        );
    }

    public function addMenuPostCount( $menu ) {
        $count = wp_count_posts(  self::$postTypeName, 'readable' );
        $count = $count->publish;

        foreach($menu as $index => $item){
            if( $item[2] === 'edit.php?post_type=' . self::$postTypeName){
                $item[0] .= '<span class="wpis_adminPostCount">' . $count . '</span>' ;
                $menu[$index] = $item;
            }
        }

        return $menu;
    }

    public function setCustomColumns($columns) {

        $columns['thumbnail'] = __( 'Image', 'wpis' );
        $columns['type-category'] = __( 'Catégorie', 'wpis' );
        $columns['price'] = __( 'Prix', 'wpis' );
        $columns['purpose'] = __( 'Raison', 'wpis' );
        $columns['status'] = __( 'Status', 'wpis' );
        $columns['reference'] = __( 'Référence', 'wpis' );
        $columns['locked'] = '<i class="fa fa-lock"></i>';
        $columns['undeletable'] = '<i class="fa fa-trash"></i>';

        $columnClone = $columns;

        $order = [
            'cb',
            'thumbnail',
            'title',
            'type-category',
            'price',
            'purpose',
            'status',
            'reference',
            'locked',
            'undeletable',
            'date'
        ];

        usort($columns, function ($a, $b) use ($order) {
            if( isset($columns) ){
                $pos_a = array_search($columns[$a], $order);
                $pos_b = array_search($columns[$b], $order);
                return $pos_a - $pos_b;
            }
        });

        $columnsMapped = [];

        foreach( $order as $i => $value ){
            $columnsMapped[$order[$i]] = $columnClone[$value];
        }

        return $columnsMapped;
    }

    public function applyCustomColumns( $column, $post_id ) {
        switch ( $column ) {

            case 'thumbnail' :
                echo get_the_post_thumbnail( $post_id, [50, 50] );
                break;

            case 'type-category' :
                $type = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'category_label' , true );
                $category = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'subcategory_label' , true );
                $typeCategory = Tools::ucfirstUtf8($type) . ' / ' . Tools::ucfirstUtf8($category);
                echo $typeCategory;
                break;

            case 'price' :
                $price = round(get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_price' , true ));
                $price = number_format_i18n($price);
                $price = str_replace(',', ' ', $price);
                $currency = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_currency' , true );
                echo $price . ' '  . $currency;
                break;

            case 'purpose' :
                $purpose = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'purpose_label' , true );
                $purpose = Tools::ucfirstUtf8($purpose);
                echo $purpose;
                break;

            case 'status' :
                $purpose = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'status_label' , true );
                $purpose = Tools::ucfirstUtf8($purpose);
                echo $purpose;
                break;

            case 'reference' :
                $reference = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'reference' , true );
                echo $reference;
                break;

            case 'locked' :
                $isLocked = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'locked' , true );
                echo $isLocked ? '<i class="icon-lock-on fas fa-lock"></i>' : '<i class="icon-lock-off fas fa-lock-open"></i>';
                break;

            case 'undeletable' :
                $isUndeletable = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'undeletable' , true );
                echo $isUndeletable ? '<i class="icon-undeletable-off fas fa-times-circle"></i>' : '<i class="icon-undeletable-on fas fa-check-circle"></i>';
                break;
        }
    }

    private function initTaxonomies() {
        self::$taxonomies = [
            [
                'slug'         => 'estate_cities',
                'single_name'  => 'Ville',
                'plural_name'  => 'Villes',
                'rewrite'      => ['slug' => SettingsManager::get('taxonomyEstateCitySlug', 'city')],
            ],
            [
                'slug'         => 'estate_categories',
                'single_name'  => 'Type de bien',
                'plural_name'  => 'Types de bien',
                'rewrite'      => ['slug' => SettingsManager::get('taxonomyEstateCategorySlug', 'category')],
            ],
            [
                'slug'         => 'estate_purposes',
                'single_name'  => 'Type de transaction',
                'plural_name'  => 'Types de transaction',
                'rewrite'      => ['slug' => SettingsManager::get('taxonomyEstatePurposeSlug', 'type')],
            ],
            [
                'slug'         => 'estate_ammeneties',
                'single_name'  => 'Équipement',
                'plural_name'  => 'Équipements',
                'rewrite'      => ['slug' => SettingsManager::get('taxonomyEstateAmenetySlug', 'ameneties')],
            ]
        ];
    }

    public function registerTaxonomies() {
        foreach( self::$taxonomies as $taxonomy ){
            $taxonomy['post_type'] = self::$postTypeName;

            $labels = array(
                'name' => $taxonomy['plural_name'],
                'singular_name' => $taxonomy['single_name'],
                'search_items' =>  'Search ' . $taxonomy['plural_name'],
                'all_items' => 'All ' . $taxonomy['plural_name'],
                'parent_item' => 'Parent ' . $taxonomy['single_name'],
                'parent_item_colon' => 'Parent ' . $taxonomy['single_name'] . ':',
                'edit_item' => 'Edit ' . $taxonomy['single_name'],
                'update_item' => 'Update ' . $taxonomy['single_name'],
                'add_new_item' => 'Add New ' . $taxonomy['single_name'],
                'new_item_name' => 'New ' . $taxonomy['single_name'] . ' Name',
                'menu_name' => $taxonomy['plural_name']
            );

            $rewrite = isset( $taxonomy['rewrite'] ) ? $taxonomy['rewrite'] : array( 'slug' => $taxonomy['slug'] );
            $hierarchical = isset( $taxonomy['hierarchical'] ) ? $taxonomy['hierarchical'] : true;

            register_taxonomy( $taxonomy['slug'], $taxonomy['post_type'], array(
                'hierarchical' => $hierarchical,
                'labels' => $labels,
                'show_ui' => true,
                'query_var' => true,
                'rewrite' => $rewrite,
            ));
        }
    }

    public function handleRedirection() {

        add_action( 'parse_request', function( $wp ){
            if ( preg_match( '#^wpis-estate-id/(.*)#', $wp->request, $matches ) ) {
                $id = $matches[1];

                $result = self::$db->get_row(
                    self::$db->prepare( 'SELECT post_id FROM ' . self::$db->postmeta . ' WHERE meta_key = "wpis_id" AND meta_value = %s',
                    $id
                ));

                if( ! $result ){
                    global $wp_query;
                    $wp_query->set_404();
                    status_header(404);
                    exit;
                }

                $permalink = get_permalink($result->post_id);
                wp_redirect( $permalink, 301 );

                exit;
            }
        } );

    }

    public function createMetabox(){

        // Estate Properties metabox
        foreach(self::$postTypeMetas as $group) {
            if( $group['id'] === 'hidden' ){
                continue;
            }
            $label = '<span class="estateMetabox__label"><i class="estateMetabox__label_icon fas ' . $group['icon'] . '"></i>%s</span>';
            add_meta_box(
                self::$postTypeMetaPrefix . '_'  . $group['id'],
                sprintf($label, $group['label']),
                function($post) use ($group) {
                    return $this->generateMetaboxBody($post, $group['id']);
                },
                [self::$postTypeName, ProjectPostType::$postTypeName],
                'side',
                'low',
                ['__back_compat_meta_box' => false]
            );
        }

        // Gallery metabox
        $label = '<span class="estateMetabox__label"><i class="estateMetabox__label_icon fas fa-images"></i>%s</span>';
        add_meta_box(
            self::$postTypeMetaPrefix . '_'  . 'pictures',
            sprintf($label, 'Images du bien'),
            [$this, 'generateMetaboxBodyGallery'],
            [self::$postTypeName, ProjectPostType::$postTypeName],
            'normal',
            'high'
        );
    }

    private function generateMetaboxBody( $post, $groupId ){
        foreach(self::$postTypeMetas as $group) {
            if( $group['id'] === $groupId ){
                echo '<div class="wpis_options_wrapper">';
                foreach($group['fields'] as $field){
                    $field = str_replace('.', '_', $field);

                    $value = get_post_meta($post->ID, self::$postTypeMetaPrefix . '_' . $field['field'], true);
                    $labelHtml =
                        '<p class="post-attributes-label-wrapper">' .
                        '   <label class="post-attributes-label" for="parent_id">' . $field['label'] . '</label>' .
                        '</p>';

                    if( $field['type'] === 'readonly' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            disabled="disabled"
                            type="text"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'text' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            type="text"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'number' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . round($value) .'"
                            type="number"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'date' ) {
                        $value = substr($value, 0, 10);
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            type="date"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'boolean' ) {
                        echo '<section class="slider-checkbox">';
                        echo '  <input type="checkbox" ' . ($value ? 'checked' : '') . ' id="' . $field['field'] . '" name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '" />';
                        echo '  <label class="label" for="' . $field['field'] . '">' . $field['label'] . '</label>';
                        echo '</section>';
                    }

                    if( $field['type'] === 'file-collection' ) {
                        foreach($value as $file) {
                            echo '<a href="' . $file->url . '" target="_blank">';
                            echo $file->filename . '.' . $file->format;
                            echo '</a>';
                        }
                    }
                }
                echo '</div>';
            }
        }
    }

    public function generateMetaboxBodyGallery( $post ) {
        $medias = get_attached_media('image', $post->ID);

        echo '<ul id="estateMetaboxPictures" class="estateMetabox__pictures">';
        foreach ($medias as $media) {
            echo '<li>';
            echo '  <figure>';
            echo wp_get_attachment_image($media->ID);
            echo '  <i class="fa fa-times"></i>';
            echo '  </figure>';
            echo '</li>';
        }
        echo '</ul>';
        echo "<script>jQuery(function($){

            // Set all variables to be used in scope
            var frame,
                metaBox = $('#estateMetaboxPictures'), // Your meta box id here
                addImgLink = metaBox.find('.upload-custom-img'),
                delImgLink = metaBox.find( '.delete-custom-img'),
                imgContainer = $('#estateMetaboxPictures'),
                imgIdInput = metaBox.find( '.custom-img-id' );

            // ADD IMAGE LINK
            addImgLink.on( 'click', function( event ){

              event.preventDefault();

              // If the media frame already exists, reopen it.
              if ( frame ) {
                frame.open();
                return;
              }

              // Create a new media frame
              frame = wp.media({
                title: 'Select or Upload Media Of Your Chosen Persuasion',
                button: {
                  text: 'Use this media'
                },
                multiple: false  // Set to true to allow multiple files to be selected
              });


              // When an image is selected in the media frame...
              frame.on( 'select', function() {

                // Get media attachment details from the frame state
                var attachment = frame.state().get('selection').first().toJSON();

                // Send the attachment URL to our custom image input field.
                imgContainer.append( '<img src=\"'+attachment.url+'\" style=\"max-width:100%;\"/>' );

                // Send the attachment id to our hidden input
                imgIdInput.val( attachment.id );

                // Hide the add image link
                addImgLink.addClass( 'hidden' );

                // Unhide the remove image link
                delImgLink.removeClass( 'hidden' );
              });

              // Finally, open the modal on click
              frame.open();
            });


            // DELETE IMAGE LINK
            delImgLink.on( 'click', function( event ){

              event.preventDefault();

              // Clear out the preview image
              imgContainer.html( '' );

              // Un-hide the add image link
              addImgLink.removeClass( 'hidden' );

              // Hide the delete image link
              delImgLink.addClass( 'hidden' );

              // Delete the image id from the hidden input
              imgIdInput.val( '' );

            });

          });</script>";
    }

    public function saveMetaboxData( $post_id, $post ) {

        $post_type = get_post_type_object( $post->post_type );

        if ( isset($post_type->cap) && !current_user_can( $post_type->cap->edit_post, $post_id ) )
            return $post_id;

        foreach(self::$postTypeMetas as $group) {
            if( $group['id'] !== 'hidden' ){
                foreach($group['fields'] as $field){
                    if( !empty($_POST) ){
                        $meta = self::$postTypeMetaPrefix . '_' . str_replace('.', '_', $field['field']);
                        if( $field['type'] === 'boolean'){
                            $value = (isset( $_POST[$meta] ) && $_POST[$meta] === 'on') ? 1 : 0;
                            update_post_meta( $post_id, $meta, $value );
                        }
                        elseif( isset( $_POST[$meta] ) ) {
                            update_post_meta( $post_id, $meta, $_POST[$meta] );
                        }
                    }
                }
            }
        }
    }
}